/***********************************************************************

FILE
    main1.c  -  d.c. motor speed control example, main program module 1

ROUTINES
    main    -  main program
    cmdint  -  command processor

REMARKS
    This program module, together with cntrl0.c and rtsim0.c implements
    simple control of the speed of a d.c. motor.  This is the main
    program module and it implements a command processor.
    
    The other two module implements the P control algorithm (cntrl1.c)
    and a simulation of the motor set-up (rtsim1.c).

    To generate the simulation program under Microsoft C version 4.00
    and above, type: cl /DSIMRT main1.c cntrl1.c rtsim1.c

    This version differs from main0.c in that its command processor
    includes commands to set the clock and sample interval.

LAST UPDATE
    18 January 1988
        minor maintenance

    Copyright (c) 1984-1988  D.M. Auslander and C.H. Tham

***********************************************************************/

/***********************************************************************
                            I M P O R T S
***********************************************************************/

#include <stdio.h>

#include "envir.h"          /* program environment declarations */
#include "cntrl1.h"         /* extern declarations for cntrl1.c */


/***********************************************************************
             F O R W A R D    D E C L A R A T I O N S
***********************************************************************/

#ifdef ANSI

void cmdint(char *);
void menu(void);

#else

void cmdint();
void menu();

#endif


/***********************************************************************
    P R I V A T E    D A T A     A N D    D E C L A R A T I O N S
***********************************************************************/

#define PROMPT  "READY "    /* defines the command processor prompt */
#define BUFSIZE 100         /* command input buffer size */


/***********************************************************************
                  E N T R Y    R O U T I N E S
***********************************************************************/

main()
{
    char cbuf[BUFSIZE];     /* space for the user-typed command line */


    menu();             /* print menu of commands */

    for (;;)            /* an endless loop. */
    {
        fputs(PROMPT, stdout);          /* print a prompt */

        fgets(cbuf, BUFSIZE, stdin);    /* get command line from user */

        cmdint(cbuf);                   /* interpret and execute commands */
    }
}



/***********************************************************************
                P R I V A T E     R O U T I N E S
***********************************************************************/

/*----------------------------------------------------------------------
PROCEDURE
    cmdint  -  command interpreter

SYNOPSIS
    static void cmdint(cline)
    char cline[];

PARAMETERS
    cline  -  user's command line (a null terminated character array)

DESCRIPTION
    Since single-letter commands are used, the first character of the
    command line denotes the command.  A switch statement is used to
    decide which command is specified.  The rest of the command line
    is then passed to appropriate routines which extract the arguments
    (if any) to the command.

LAST UPDATE
    18 March 1985
----------------------------------------------------------------------*/

static void cmdint(cline)
char cline[];
{

    switch(cline[0])        /* Execute different sections of code  */
    {                       /*   depending on 1st char of cline[]  */
        case 'k':   
                                /* set controller gain, the remainder */
            gain(&cline[1]);    /*  of the string is sent to the      */
                                /*  function gain() for decoding      */
            break;

        case 's':   
        
            setv(&cline[1]);    /* set the setpoint */
            break;

        case 'g':   
        
            control(&cline[1]); /* "go," i.e., start control. */
            break;

        case 'i':
        
            init(&cline[1]);    /* initialize the system */
            break;

        case 't':
    
            smpset(&cline[1]);  /* set sample interval. */
            break;

        case '?':
        case 'h':               /* "help," print the command list */

            menu();             /* print menu of commands */
            break;

        case 'e':   
        
            exit(0);            /* Return to operating system. */
            break;

        default:                /* no match found */
        
            printf("command not found: %s\n", cline);
    }

}



/*----------------------------------------------------------------------
PROCEDURE
    MENU  -  print a reminder of commands

SYNOPSIS
    static void menu()

LAST UPDATE
    17 November 1987
----------------------------------------------------------------------*/

static void menu()
{

    printf("\nCommand List Usage: command [argument(s)]\n\n");

    printf("command  arguments        remarks\n");
    printf("-------  ---------        -------\n");
    printf(" h                        : print this menu\n");
    printf(" i       delta_t          : initialize\n");
    printf(" k       kp ki const      : set controller gains\n");
    printf(" s       setpoint         : specify setpoint\n");
    printf(" t       sample_interval  : set sample interval\n");
    printf(" g       iterations prnt  : start\n");
    printf(" e                        : exit\n");

    printf("\n");

}
